#include "stdafx.h"
#include "messages.h"
#include <sstream>
#include <iostream>

static void DisplayText(WORD colour, const std::string& str)
{
	HANDLE hStd = GetStdHandle(STD_ERROR_HANDLE);
	BOOL conAttributesChanged = FALSE;
	CONSOLE_SCREEN_BUFFER_INFO csbi = {0};
	if(GetConsoleScreenBufferInfo(hStd, &csbi))
	{
		conAttributesChanged = SetConsoleTextAttribute(hStd, colour);
	}
	DWORD written = 0;
	WriteConsoleA(hStd, str.c_str(), static_cast<DWORD>(str.length()), &written, NULL);
	if(conAttributesChanged)
	{
		SetConsoleTextAttribute(hStd, csbi.wAttributes);
	}
}

void DisplayError(const std::string& str)
{
	DisplayText(FOREGROUND_RED | FOREGROUND_INTENSITY, str);
}

void DisplayStatus(const std::string& str)
{
	DisplayText(FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE | FOREGROUND_INTENSITY, str);
}

std::string GetModuleMessage(HMODULE hMod, DWORD messageId)
{
	DWORD flags = FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_IGNORE_INSERTS;
	flags |= hMod ? FORMAT_MESSAGE_FROM_HMODULE : FORMAT_MESSAGE_FROM_SYSTEM;
	LPSTR error;
	std::string errorText;
	if(FormatMessageA(
			flags,
			hMod,
			messageId,
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
			reinterpret_cast<LPSTR>(&error),
			0,
			NULL
		)
	)
	{
		errorText = error;
		LocalFree(error);
	}
	return errorText;
}

static std::string GetErrorMessage(DWORD err)
{
	HMODULE hMod = NULL;
	// this is probably an NTSTATUS
	if(err & 0x80000000)
	{
		// redirect access denied, because the ntdll message
		// makes it sound like a memory error, when its more likely
		// a permissions one
		if(err == STATUS_ACCESS_DENIED)
		{
			return "Access Denied";
		}
		hMod = GetModuleHandle(L"ntdll.dll");
	}
	return GetModuleMessage(hMod, err);
}

void DisplayLastError(const char* pString)
{
	DisplayError(pString, GetLastError());
}

// chroot directory [options] program args
// options can be
// -h (help)
// -v (version)
// -a (run as anonymous user) (creates a user, runs the program, deletes the user)
// -u <userName> (run as a different user)
// (-a and -u cannot be combined)
// (if they are both absent the command is run as the one who started chroot)
void PrintHelp()
{
	const char* pUsage = 
		"Usage: chroot rootDir [options] program args\n"
		"Where [options] can be none, one, or more of these:\n"
		"-h            Shows this help\n"
		"-v            Shows the version banner\n"
		"-a            Run 'program' as an anonymous user\n"
		"-u <userName> Run 'program' as user <userName>\n"
		"-f <letters>  Allow 'program' to access the drives identified in <letters>\n"
		"\n"
		"Note -a and -u cannot both be defined. If neither is defined\n"
		"'program' will be run as the user running chroot\n"
		"\n"
		"<letters> should be a collection of drive letters, e.g.\n"
		"to allow 'program' to access drives C: and D:\n"
		"the format of the option would be -f cd\n"
		"\n"
		"'rootDir' and 'program' can contain environment variables";
	std::cout << pUsage << std::endl;;
}
