#ifndef CHROOT_STDAFX_H
#define CHROOT_STDAFX_H

#pragma once

#define NOMINMAX
#define _WIN32_WINNT 0x0501
#define WIN32_LEAN_AND_MEAN
#define _SCL_SECURE_NO_WARNINGS

#include "simpleassert.h"
#include <ntstatus.h>
#define WIN32_NO_STATUS

// why, oh why are these two not defined in ntstatus.h
__if_not_exists(NTSTATUS)
{
	typedef long NTSTATUS;
}

#ifndef NT_SUCCESS
#define NT_SUCCESS(Status) (((NTSTATUS)(Status)) >= 0)
#endif

#include <windows.h>
#include <strsafe.h>
#include "scopedobjects.h"
#include <string>
#include <limits>

// only pass narrow literals for the 'string' parameter
#define ERROR_TEXT(string, ...) \
	do \
	{ \
		C_ASSERT(sizeof(*string) == sizeof(char)); \
		char buffer[500]; \
		StringCchPrintfA(buffer, ARRAYSIZE(buffer), __FUNCTION__ " " string "\n", __VA_ARGS__); \
		OutputDebugStringA(buffer); \
	} \
	while(0)

#if defined(_DEBUG) || defined(DBG) || defined(_DBG)
#define DBG_TEXT(string, ...) ERROR_TEXT(string, __VA_ARGS__)
#else
#define DBG_TEXT(...)
#endif

struct CmdLineArgs
{
	// args of the program to chroot
	LPWSTR* ppProgArgs;
	// the location of the chroot jail
	std::wstring dir;
	// username given by user gotten when
	std::wstring userName;
	// password of user
	std::wstring password;
	// number of args in ppProgArgs
	int numProcArgs;
	// drives to not redirect
	// this is a bitmask, valid indexes are 0 (A:) to 25 (Z:)
	DWORD drivesToKeep;
	// whether the user wants to run the prog as 
	// a different, but existing user
	bool promptForUser;
	// whether we should create a user to run the program
	bool createUser;
};

std::wstring ExpandEnvString(LPCWSTR pStr);
WCHAR FindARootedDriveLetter(DWORD keptDriveMask);

#endif
