#ifndef CHROOT_UNDECLARED_H
#define CHROOT_UNDECLARED_H

#pragma once

typedef enum _RTL_PATH_TYPE
{
	RtlPathTypeUnknown,
	RtlPathTypeUncAbsolute,
	RtlPathTypeDriveAbsolute,
	RtlPathTypeDriveRelative,
	RtlPathTypeRooted,
	RtlPathTypeRelative,
	RtlPathTypeLocalDevice,
	RtlPathTypeRootLocalDevice
} RTL_PATH_TYPE;

typedef RTL_PATH_TYPE (NTAPI*pfnRtlDetermineDosPathNameType_U)(
	PCWSTR DosFileName
);

//++
//
// VOID
// InitializeObjectAttributes(
//     OUT POBJECT_ATTRIBUTES p,
//     IN PUNICODE_STRING n,
//     IN ULONG a,
//     IN HANDLE r,
//     IN PSECURITY_DESCRIPTOR s
//     )
//
//--

#ifndef InitializeObjectAttributes
#define InitializeObjectAttributes( p, n, a, r, s ) { \
    (p)->Length = sizeof( OBJECT_ATTRIBUTES );          \
    (p)->RootDirectory = r;                             \
    (p)->Attributes = a;                                \
    (p)->ObjectName = n;                                \
    (p)->SecurityDescriptor = s;                        \
    (p)->SecurityQualityOfService = NULL;               \
    }
#endif

//
// Valid values for the Attributes field
//

#define OBJ_INHERIT             0x00000002L
#define OBJ_PERMANENT           0x00000010L
#define OBJ_EXCLUSIVE           0x00000020L
#define OBJ_CASE_INSENSITIVE    0x00000040L
#define OBJ_OPENIF              0x00000080L
#define OBJ_OPENLINK            0x00000100L
#define OBJ_KERNEL_HANDLE       0x00000200L
#define OBJ_FORCE_ACCESS_CHECK  0x00000400L

typedef struct _UNICODE_STRING {
    USHORT Length;
    USHORT MaximumLength;
    PWSTR  Buffer;
} UNICODE_STRING;
typedef UNICODE_STRING *PUNICODE_STRING;
typedef const UNICODE_STRING *PCUNICODE_STRING;

typedef struct _OBJECT_ATTRIBUTES {
    ULONG Length;
    HANDLE RootDirectory;
    PUNICODE_STRING ObjectName;
    ULONG Attributes;
    PVOID SecurityDescriptor;
    PVOID SecurityQualityOfService;
} OBJECT_ATTRIBUTES;
typedef OBJECT_ATTRIBUTES *POBJECT_ATTRIBUTES;

#define DIRECTORY_QUERY                 (0x0001)
#define DIRECTORY_TRAVERSE              (0x0002)
#define DIRECTORY_CREATE_OBJECT         (0x0004)
#define DIRECTORY_CREATE_SUBDIRECTORY   (0x0008)

#define DIRECTORY_ALL_ACCESS (STANDARD_RIGHTS_REQUIRED | 0xF)

typedef NTSTATUS (NTAPI*pfnNtCreateDirectoryObject)(
	OUT PHANDLE pDirectoryHandle,
	IN ACCESS_MASK DesiredAccess,
	IN POBJECT_ATTRIBUTES pObjectAttributes
);

typedef VOID (NTAPI*pfnRtlSetLastWin32ErrorAndNtStatusFromNtStatus)(NTSTATUS error);

typedef enum __PROCESSINFOCLASS 
{
	ProcessBasicInformation,
	ProcessQuotaLimits,
	ProcessIoCounters,
	ProcessVmCounters,
	ProcessTimes,
	ProcessBasePriority, // invalid for query
	ProcessRaisePriority, // invalid for query
	ProcessDebugPort,
	ProcessExceptionPort, // invalid for query
	ProcessAccessToken, // invalid for query
	ProcessLdtInformation,
	ProcessLdtSize, // invalid for query
	ProcessDefaultHardErrorMode,
	ProcessIoPortHandlers,          // Note: this is kernel mode only, invalid for query
	ProcessPooledUsageAndLimits,
	ProcessWorkingSetWatch,
	ProcessUserModeIOPL, // invalid class
	ProcessEnableAlignmentFaultFixup, // invalid class
	ProcessPriorityClass,
	ProcessWx86Information,
	ProcessHandleCount, 
	ProcessAffinityMask, // invalid for query
	ProcessPriorityBoost,
	ProcessDeviceMap,
	ProcessSessionInformation,
	ProcessForegroundInformation, // invalid for query
	ProcessWow64Information,
	ProcessImageFileName,
	ProcessLUIDDeviceMapsEnabled,
	ProcessBreakOnTermination,
	ProcessDebugObjectHandle,
	ProcessDebugFlags, // EProcess->Flags.NoDebugInherit
	ProcessHandleTracing, 
	ProcessIoPriority,
	ProcessExecuteFlags,
	ProcessTlsInformation, // invalid class
	ProcessCookie,
	ProcessImageInformation, // last available on XPSP3
	ProcessCycleTime,
	ProcessPagePriority,
	ProcessInstrumentationCallback, // invalid class
	ProcessThreadStackAllocation, // invalid class
	ProcessWorkingSetWatchEx,
	ProcessImageFileNameWin32, // buffer is a UNICODE_STRING
	ProcessImageFileMapping, // buffer is a pointer to a file handle open with SYNCHRONIZE | FILE_EXECUTE access, return value is whether the handle is the same used to start the process
	ProcessAffinityUpdateMode,
	ProcessMemoryAllocationMode,
	ProcessGroupInformation,
    ProcessTokenVirtualizationEnabled, // invalid class
    ProcessConsoleHostProcess, // retrieves the pid for the process' corresponding conhost process
    ProcessWindowInformation, // returns the windowflags and windowtitle members of the process' peb->rtl_user_process_params
	MaxProcessInfoClass // MaxProcessInfoClass should always be the last enum
} PROCESSINFOCLASS;


typedef NTSTATUS (NTAPI*pfnNtSetInformationProcess)(
	HANDLE ProcessHandle,
	PROCESSINFOCLASS ProcessInformationClass,
	PVOID pProcessInformation,
	ULONG ProcessInformationLength
);

//
// Process Device Map information
//  NtQueryInformationProcess using ProcessDeviceMap
//  NtSetInformationProcess using ProcessDeviceMap
//

typedef struct _PROCESS_DEVICEMAP_INFORMATION {
    union {
        struct {
            HANDLE DirectoryHandle;
        } Set;
        struct {
            ULONG DriveMap;
            UCHAR DriveType[ 32 ];
        } Query;
    } DUMMYUNIONNAME;
} PROCESS_DEVICEMAP_INFORMATION, *PPROCESS_DEVICEMAP_INFORMATION;

#define SYMBOLIC_LINK_QUERY (0x0001)
#define SYMBOLIC_LINK_ALL_ACCESS (STANDARD_RIGHTS_REQUIRED | 0x1)

typedef NTSTATUS (NTAPI*pfnNtCreateSymbolicLinkObject)(
    OUT PHANDLE pLinkHandle,
    IN ACCESS_MASK DesiredAccess,
    IN POBJECT_ATTRIBUTES pObjectAttributes,
    IN PUNICODE_STRING pLinkTarget
);

extern pfnNtCreateSymbolicLinkObject NtCreateSymbolicLinkObject;
extern pfnRtlDetermineDosPathNameType_U RtlDetermineDosPathNameType_U;
extern pfnNtSetInformationProcess NtSetInformationProcess;
extern pfnNtCreateDirectoryObject NtCreateDirectoryObject;
extern pfnRtlSetLastWin32ErrorAndNtStatusFromNtStatus RtlSetLastWin32ErrorAndNtStatusFromNtStatus;

BOOL InitNtDllFnPtrs();

#endif
