#include "stdafx.h"
#include "unpack.h"
#include "resource.h"
#include <cstring>
#include <cstdio>
#include <cstdlib>
#include <process.h>
#include "messages.h"

static BOOL Unpack64BitVersion(std::wstring& newFileLocation)
{
	// could just have used GMH(null)
	// but if it will be later spun
	// into a library that might be wrong
	HMODULE hMod = NULL;
	if(!GetModuleHandleEx(
		GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS | GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT,
		reinterpret_cast<LPWSTR>(&Unpack64BitVersion),
		&hMod)
	)
	{
		return FALSE;
	}
	HRSRC hRes = FindResource(hMod, MAKEINTRESOURCE(SIXTYFOURBIT_RESOURCE_ID), RT_RCDATA);
	if(!hRes)
	{
		ERROR_TEXT("Couldn't find required service resource - %lu", GetLastError());
		return FALSE;
	}
	HGLOBAL hGlob = LoadResource(hMod, hRes);
	PVOID pExe = LockResource(hGlob);
	ULONG exeSize = SizeofResource(hMod, hRes);
	ASSERT(pExe && exeSize);

	PWSTR pTempFileBuffer = _wtempnam(NULL, L"64b");
	if(!pTempFileBuffer)
	{
		DBG_TEXT("Couldn't generate temporary name!");
		return FALSE;
	}
	newFileLocation = pTempFileBuffer;
	free(pTempFileBuffer);
	PCWSTR pTempFile = newFileLocation.c_str();

	WinType<> hFile(
		CreateFile(
			pTempFile,
			GENERIC_WRITE,
			FILE_SHARE_DELETE,
			NULL,
			CREATE_NEW,
			FILE_ATTRIBUTE_NORMAL,
			NULL
		), 
		&CloseHandle
	);
	if(*hFile == INVALID_HANDLE_VALUE)
	{
		ERROR_TEXT("Couldn't create temporary file %S, error %lu", pTempFile, GetLastError());
		return FALSE;
	}
	DWORD bytes = 0;
	BOOL bRet = WriteFile(*hFile, pExe, exeSize, &bytes, NULL);
	if(!bRet)
	{
		ERROR_TEXT("Couldn't write temporary file, error %lu", GetLastError());
		DeleteFile(pTempFile);
	}
	return bRet;
}

int Run64BitVersion(int argc, wchar_t** argv)
{
	std::wstring pathTo64BitApp;
	if(!Unpack64BitVersion(pathTo64BitApp))
	{
		DisplayLastError("Couldn't extract native version from resources");
		return 1;
	}
	// this shouldn't be modified by anything
	argv[0] = const_cast<PWSTR>(pathTo64BitApp.c_str());
	errno = 0;
	intptr_t ret = _wspawnv(P_WAIT, argv[0], argv);
	if(ret == -1)
	{
		DWORD error = 0;
		_get_doserrno(&error);
		DisplayError("Couldn't run native version", error);
	}
	DeleteFile(pathTo64BitApp.c_str());
	return static_cast<int>(ret);
}
